import boto3
from botocore.exceptions import ClientError

TEMPLATE_FILE = 'budgets.template.yaml'


def read_cf_template():
    with open(TEMPLATE_FILE, 'r') as f:
        return f.read()


def check_if_boto_response_is_successful(response):
    return response.get('ResponseMetadata', {}).get('HTTPStatusCode',
                                                    None) == 200


class CloudFormationClient():

    def __init__(self,
                 admin_email,
                 budget,
                 stack_name='aws-budgets',
                 profile=None):
        if profile:
            boto3.setup_default_session(profile_name=profile)

        self.client = boto3.client('cloudformation')
        self.stack_name = stack_name
        self.admin_email = admin_email
        self.budget = budget

    def _does_budget_stack_exist(self):
        # If this succeeds, the stack exists
        try:
            response = self.client.describe_stacks(StackName=self.stack_name)
            return check_if_boto_response_is_successful(response)
        # If the stack doesn't exist, boto throws this exception
        except ClientError:
            return False

    def create_stack(self):
        """ Creates CloudFormation stack for budgets, if it does not yet exist.
            Returns True if successful
        """
        if self._does_budget_stack_exist():
            print("%s CloudFormation stack already exists!" % self.stack_name)
            return False

        response = self.client.create_stack(
            StackName=self.stack_name,
            TemplateBody=read_cf_template(),
            Parameters=[{
                'ParameterKey': 'AdminEmail',
                'ParameterValue': self.admin_email,
            }, {
                'ParameterKey': 'BudgetThreshold',
                'ParameterValue': self.budget,
            }])

        if not check_if_boto_response_is_successful(response):
            print("Failed to create CloudFormation stack!")
            return False

        return response.get('StackId')



import click
from CloudFormationClient import CloudFormationClient


def generate_new_budget(admin_email, budget, profile):
    if admin_email is None:
        admin_email = click.prompt(
                click.style(
                    "Please enter the email address where you'd like budget notifications sent",
                    bold=True),
                type=str)

    click.echo(f'Sending notifications to {click.style(admin_email, fg="green")}')

    if budget is None:
        budget = click.prompt(
                click.style("Enter your budget in USD", bold=True), type=str)

    click.echo(f'Creating a new budget of {click.style("$" + budget, fg="green")}')

    # Create a new CloudFormation client to manage budgets
    cfclient = CloudFormationClient(admin_email, budget, profile=profile)
    stackid = cfclient.create_stack()
    if stackid:
        click.echo(f'Created CloudFormation stack to manage budgets: {stackid}')


@click.command()
@click.option('--new', '-n', is_flag=True, help="Create a new AWS budget")
@click.option('--admin_email', '-e', default=None, help="The email address to which budget notifications are sent")
@click.option('--budget', '-b', default=None, help="The budget (in USD) you want to set")
@click.option('--profile', '-p', default=None, help="AWS profile name")
def cli(new, admin_email, budget, profile):
    if new is True:
        generate_new_budget(admin_email, budget, profile)
        return



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="awsbudget",
    description='Tool to manage a single, simple AWS Budget',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords=['aws', 'budget'],
    version='0.0.4',
    author='Dylan Sather',
    author_email='dylan.sather@gmail.com',
    url='https://github.com/dylburger/aws-budgets-cli',
    license='MIT',
    python_requires='>=3',
    py_modules=['awsbudget'],
    packages=find_packages(),
    install_requires=[
        'boto3',
        'click',
    ],
    entry_points='''
        [console_scripts]
        awsbudget=awsbudget:cli
    ''',
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
    )
)
